<?php

namespace Drupal\intercept_core\Form;

use Drupal\Core\Ajax\AjaxFormHelperTrait;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\intercept_core\Entity\ReservationInterface;

/**
 * A form to update the status of a reservation.
 */
class ReservationStatusChangeForm extends FormBase {

  use AjaxFormHelperTrait;

  /**
   * The status field name.
   *
   * @var string
   */
  protected $fieldName = 'field_status';

  /**
   * The options array.
   *
   * @var array
   */
  protected $options;

  /**
   * The reservation entity.
   *
   * @var \Drupal\intercept_core\Entity\ReservationInterface
   */
  protected $entity;

  /**
   * Sets the URL options.
   *
   * @param array $options
   *   The array of options. See \Drupal\Core\Url::fromUri() for details on what
   *   it contains.
   *
   * @return $this
   */
  public function setOptions(array $options) {
    $this->options = $options;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    return 'reservation_status_change_form';
  }

  /**
   * {@inheritdoc}
   */
  public function setEntity(ReservationInterface $entity) {
    $this->entity = $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    $entity = $this->entity;
    if (!$entity) {
      throw new \RuntimeException('No entity provided to ReservationStatusChangeForm.');
    }
    $form_id = $this->getBaseFormId();
    $form_id .= '_' . $entity->getEntityTypeId() . '_' . $this->fieldName;
    $form_id .= '_' . $entity->id();

    return $form_id;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\intercept_core\Entity\ReservationInterface $entity */
    $entity = $this->entity;

    $options = $entity->getStatusOperations();
    $current_value = $entity->get($this->fieldName)->value;
    $wrapper_id = 'js--' . $this->getFormId() . '__wrapper';

    $form['current_status'] = [
      '#theme' => 'intercept_reservation_status',
      '#status' => $current_value,
    ];
    $form['#prefix'] = '<div id="' . $wrapper_id . '">';
    $form['#suffix'] = '</div>';
    $form['actions'] = [
      '#type' => 'actions',
      '#attributes' => [
        'class' => [
          'form-actions--operations',
        ],
      ],
    ];

    foreach ($options as $op) {
      $form['actions'][$op] = [
        '#type' => 'submit',
        '#value' => $op,
        '#op' => $op,
        '#ajax' => [
          'callback' => [$this, 'successfulAjaxSubmit'],
          'wrapper' => $wrapper_id,
        ],
        '#attributes' => [
          'class' => [
            'use-ajax-submit',
          ],
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $op = $triggering_element['#op'];

    /** @var \Drupal\intercept_core\Entity\ReservationInterface $entity */
    $entity = $this->entity;
    $entity->{$op}();
    $entity->save();
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function successfulAjaxSubmit(array $form, FormStateInterface $form_state) {
    // TODO: Consider replacing this with a custom Ajax response so the form does not reload with
    // all the stored Drupal messages.
    $response = new AjaxResponse();
    $response->addCommand(new HtmlCommand(NULL, $form));

    // Trigger the Save success event.
    $command = new InvokeCommand('html', 'trigger', [
      'intercept:saveRoomReservationSuccess',
    ]);
    $response->addCommand($command);
    return $response;
  }

}
